From 1cd2b77d6840b22e0589de9216a00410719ac8bb Mon Sep 17 00:00:00 2001
From: Yang Hamo Bai <hamo.by@gmail.com>
Date: Wed, 29 Jun 2011 20:20:44 +0800
Subject: [PATCH 01/12] xcompile: added ARM toolchain support

Till now, we only support arm-none-eabi, using
others will be supported in the future.

Signed-off-by: Yang Hamo Bai <hamo.by@gmail.com>
---
 util/xcompile/xcompile |  126 +++++++++++++++++++++++++----------------------
 1 files changed, 67 insertions(+), 59 deletions(-)

diff --git a/util/xcompile/xcompile b/util/xcompile/xcompile
index 19103c7..276878c 100644
--- a/util/xcompile/xcompile
+++ b/util/xcompile/xcompile
@@ -3,6 +3,8 @@
 # This file is part of the coreboot project.
 #
 # Copyright (C) 2007-2010 coresystems GmbH
+# Copyright (C) 2010 Google Inc
+# Copyright (C) 2011 Yang Bai <hamo.by@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,69 +36,75 @@ for make in make gmake gnumake; do
 	fi
 done
 
-GCCPREFIX=invalid
-TMPFILE=`mktemp /tmp/temp.XXXX 2>/dev/null || echo /tmp/temp.78gOIUGz`
-touch $TMPFILE
-
-# This should be a loop over all supported architectures
-TARCH=i386
 TWIDTH=32
-for gccprefixes in `pwd`/util/crossgcc/xgcc/bin/${TARCH}-elf- ${TARCH}-elf- ""; do
-	if ! which ${gccprefixes}as 2>/dev/null >/dev/null; then
-		continue
-	fi
-	rm -f ${TMPFILE}.o
-	if ${gccprefixes}as -o ${TMPFILE}.o ${TMPFILE}; then
-		TYPE=`${gccprefixes}objdump -p ${TMPFILE}.o`
-		if [ ${TYPE##* } == "elf${TWIDTH}-${TARCH}" ]; then
-			GCCPREFIX=$gccprefixes
-			ASFLAGS=
-			CFLAGS=
-			LDFLAGS=
-			break
-		fi
-	fi
-	if ${gccprefixes}as --32 -o ${TMPFILE}.o ${TMPFILE}; then
-		TYPE=`${gccprefixes}objdump -p ${TMPFILE}.o`
-		if [ ${TYPE##* } == "elf${TWIDTH}-${TARCH}" ]; then
-			GCCPREFIX=$gccprefixes
-			ASFLAGS=--32
-			CFLAGS="-m32 "
-			LDFLAGS="-b elf32-i386"
-			break
-		fi
-	fi
-done
-rm -f $TMPFILE ${TMPFILE}.o
-
-if [ "$GCCPREFIX" = "invalid" ]; then
-	echo '$(error no suitable gcc found)'
-	exit 1
-fi
+for TARCH in i386 littlearm; do
 
-CC="${GCCPREFIX}gcc"
-testcc "$CC" "$CFLAGS-Wa,--divide " && CFLAGS="$CFLAGS-Wa,--divide "
-testcc "$CC" "$CFLAGS-fno-stack-protector " && CFLAGS="$CFLAGS-fno-stack-protector "
-testcc "$CC" "$CFLAGS-Wl,--build-id=none " && CFLAGS="$CFLAGS-Wl,--build-id=none "
+	GCCPREFIX=invalid
+	
+	if [ ${TARCH} == "i386" ]; then
+		CARCH=${TARCH}-elf
+	elif [ ${TARCH} == "littlearm" ]; then
+		CARCH=arm-none-eabi
+	fi
 
-if which gcc 2>/dev/null >/dev/null; then
-	HOSTCC=gcc
-else
-	HOSTCC=cc
-fi
+	TMPFILE=`mktemp /tmp/temp.XXXX 2>/dev/null || echo /tmp/temp.78gOIUGz`
+	touch $TMPFILE
 
-cat << EOF
+	for gccprefixes in `pwd`/util/crossgcc/xgcc/bin/${CARCH}- ${CARCH}- ""; do
+		if ! which ${gccprefixes}as 2>/dev/null >/dev/null; then
+			continue
+		fi
+		rm -f ${TMPFILE}.o
+		if ${gccprefixes}as -o ${TMPFILE}.o ${TMPFILE}; then
+			TYPE=`${gccprefixes}objdump -p ${TMPFILE}.o | grep "file format"`
+			if [ ${TYPE##* } == "elf${TWIDTH}-${TARCH}" ]; then
+				GCCPREFIX=$gccprefixes
+				ASFLAGS=
+				CFLAGS=
+				LDFLAGS=
+				break
+			fi
+		fi
+		if [ ${TARCH} == "i386" ]; then
+			if ${gccprefixes}as --32 -o ${TMPFILE}.o ${TMPFILE}; then
+				TYPE=`${gccprefixes}objdump -p ${TMPFILE}.o | grep "file format"`
+				if [ ${TYPE##* } == "elf${TWIDTH}-${TARCH}" ]; then
+					GCCPREFIX=$gccprefixes
+					ASFLAGS=--32
+					CFLAGS="-m32 "
+					LDFLAGS="-b elf32-i386"
+					break
+				fi
+			fi
+		fi
+	done
+	rm -f $TMPFILE ${TMPFILE}.o
+	if [ "$GCCPREFIX" != "invalid" ]; then
+		CC="${GCCPREFIX}gcc"
+		testcc "$CC" "$CFLAGS-Wa,--divide " && CFLAGS="$CFLAGS-Wa,--divide "
+		testcc "$CC" "$CFLAGS-fno-stack-protector " && CFLAGS="$CFLAGS-fno-stack-protector "
+		testcc "$CC" "$CFLAGS-Wl,--build-id=none " && CFLAGS="$CFLAGS-Wl,--build-id=none "
+		if [ ${TARCH} == "littlearm" ]; then
+			testcc "$CC" "$CFLAGS-ffixed-r8 -msoft-float -marm -mabi=aapcs-linux -mno-thumb-interwork" && CFLAGS="$CFLAGS-ffixed-r8 -msoft-float -marm -mabi=aapcs-linux -mno-thumb-interwork "
+		fi
+		cat << EOF
 # elf${TWIDTH}-${TARCH} toolchain
-AS:=${GCCPREFIX}as ${ASFLAGS}
-CC:=${GCCPREFIX}gcc ${CFLAGS}
-AR:=${GCCPREFIX}ar
-LD:=${GCCPREFIX}ld ${LDFLAGS}
-STRIP:=${GCCPREFIX}strip
-NM:=${GCCPREFIX}nm
-OBJCOPY:=${GCCPREFIX}objcopy
-OBJDUMP:=${GCCPREFIX}objdump
-
-# native toolchain
-HOSTCC:=${HOSTCC}
+AS_${TARCH}:=${GCCPREFIX}as ${ASFLAGS}
+CC_${TARCH}:=${GCCPREFIX}gcc ${CFLAGS}
+AR_${TARCH}:=${GCCPREFIX}ar
+LD_${TARCH}:=${GCCPREFIX}ld ${LDFLAGS}
+STRIP_${TARCH}:=${GCCPREFIX}strip
+NM_${TARCH}:=${GCCPREFIX}nm
+OBJCOPY_${TARCH}:=${GCCPREFIX}objcopy
+OBJDUMP_${TARCH}:=${GCCPREFIX}objdump
 EOF
+	else
+		cat <<EOF
+# elf${TWIDTH}-${TARCH} toolchain
+NO_${TARCH}_TOOLCHAIN:=1
+EOF
+	fi
 
+	unset AS CC AR LD STRIP NM OBJCOPY OBJDUMP GCCPREFIX ASFLAGS CFLAGS LDFLAGS CARCH 
+
+done
-- 
1.7.6

